#!/bin/bash

# syntax:
# DEBUG=1 ./run.sh  - turn on DEBUG mode
# ./run.sh method - show only the result for a method
#             ^------- regular expressions

which smv &>/dev/null || {
    echo error: unable to find smv
    exit 1
}

justmodel=.                 # all models
justmethod=.                # all methods
[ "$1" ] && justmethod=$1

smvout=`mktemp smvoutput.XXXXXX`
trap 'rm $smvout' INT TERM EXIT

fmt="%-22s | %9s | %15s | %10s | %s\n"

get_vars()
{
    sed -n '/^Variable order/,/^user time/p' <$smvout |
    sed '/^user time/,$d' |
    tail -n +3 |
    wc -l
}

get_relation()
{
    grep -A 1 '^transition relation' $smvout | grep 'nodes after' |
    tail -n 1 | sed 's/.*nodes after: \([0-9]*\)]/\1/'
}

get_bdd()
{
    sed -n '/^BDD nodes allocated/s/.*\.\.\.*//p' <$smvout
}

get_time()
{
    grep '^user time' <$smvout | tail -n 1 | sed 's/.*\.\.\.*//'
}

get_result()
{
    res=`sed -n '/^Model checking results/,/^Resources used/p' $smvout |
         egrep '\.\.\.(true|false)$'`
    if [ "$res" ]; then
        if echo "$res" | grep -q false; then
            echo false
        else    
            echo true
        fi
    else
        echo N/A
    fi
}

method()
{
    case "$1" in
        noext) echo no extension;;
        dinput) echo all critical ports;;
        destabil) echo destabilizers;;
        osd) echo one-step destabilizers;;
        doutput) echo clock domain outputs;;
    esac
}

(
model=proto

[[ $model =~ $justmodel ]] || continue
echo $model
printf "$fmt" method variables "trans. relation" "reach. set" "satisfied"
echo -----------------------------------------------------------------------------
for m in noext dinput destabil osd doutput; do
    [[ $m =~ $justmethod ]] || continue
    ./verify -vvv -m $m $model.smv >$smvout
    printf "$fmt" "`method $m`" `get_vars` `get_relation` `get_bdd` \
        `get_result`
    [ "$DEBUG" ] && cp $smvout $smvout.$model.$m
done
echo -----------------------------------------------------------------------------
) | tee experiment-output
